#include <math.h>
#include <cstring>
#include "mex.h"

#define NELEMS(x)  (sizeof(x) / sizeof(x[0]))

// small value, used to avoid division by zero
#define eps 0.0001

#define FIRSTDIM 0
#define SECDIM 1


// unit vectors used to compute gradient orientation (9 directions for 180 degrees)
double uu[9] = {1.0000, 
		0.9397, 
		0.7660, 
		0.500, 
		0.1736, 
		-0.1736, 
		-0.5000, 
		-0.7660, 
		-0.9397};
double vv[9] = {0.0000, 
		0.3420, 
		0.6428, 
		0.8660, 
		0.9848, 
		0.9848, 
		0.8660, 
		0.6428, 
		0.3420};

// unit vectors used to compute curvature orientation (9 directions for 360 degrees)
/*double uu9[9] = {1.0000, 
		0.7660, 
		0.1736, 
		-0.5000, 
		-0.9397,
		-0.9397,
		-0.5000,
		0.1736, 
		0.7660};
double vv9[9] = {0.0000, 
		0.6428, 
		0.9848, 
		0.8660, 
		0.3420,
		-0.3420,
		-0.8660, 
		-0.9848, 
		-0.6428};*/
		
double uu8[8] = {1.0000,
    0.7071,
    0.0000,
   -0.7071,
   -1.0000,
   -0.7071,
   -0.0000,
    0.7071};

double vv8[8] = {0.0000, 
    0.7071,
    1.0000,
    0.7071,
    0.0000,
   -0.7071,
   -1.0000,
   -0.7071};
    

static inline float min(float x, float y) { return (x <= y ? x : y); }
static inline float max(float x, float y) { return (x <= y ? y : x); }

static inline int min(int x, int y) { return (x <= y ? x : y); }
static inline int max(int x, int y) { return (x <= y ? y : x); }

float *createMxFloatArray3dim(int dim1, int dim2, int dim3, mxArray **result)
{
	int dims[3];
	dims[0] = dim1;
	dims[1] = dim2;
	dims[2] = dim3;
	*result = mxCreateNumericArray(3, dims, mxSINGLE_CLASS, mxREAL);
	
	return (float *)mxGetData(*result);
}
	
void convolve1D5(float *outimg, float *inpimg, int dim1, int dim2, float *filter, int orientation, int padding = 0)
{
	//int filterradiusL = (dimfilter-1) / 2;
	//int filterradiusR = (dimfilter-1) - filterradiusL;
	const int filterradiusL = 2+padding;
	const int filterradiusR = 2+padding;
	
	if(orientation == FIRSTDIM) {
		for(int j = 0; j < dim2; j++) {
			for(int i = 0; i < dim1; i++) {
				if(i < filterradiusL || i > dim1-1-filterradiusR) {
					outimg[j*dim1 + i] = 0;
				} else {
					outimg[j*dim1 + i] =
							inpimg[j*dim1 + i - 2] * filter[0]
						+ inpimg[j*dim1 + i - 1] * filter[1]
						+ inpimg[j*dim1 + i    ] * filter[2]
						+ inpimg[j*dim1 + i + 1] * filter[3]
						+ inpimg[j*dim1 + i + 2] * filter[4];
				}
			}
		}
	} else if(orientation == SECDIM) {
		for(int j = 0; j < dim2; j++) {
			for(int i = 0; i < dim1; i++) {
				if(j < filterradiusL || j > dim2-1-filterradiusR) {
					outimg[j*dim1 + i] = 0;
				} else {
					outimg[j*dim1 + i] =
							inpimg[i + dim1*(j - 2)] * filter[0]
						+ inpimg[i + dim1*(j - 1)] * filter[1]
						+ inpimg[i + dim1*(j    )] * filter[2]
						+ inpimg[i + dim1*(j + 1)] * filter[3]
						+ inpimg[i + dim1*(j + 2)] * filter[4];
				}
			}
		}
	} else {
		mexErrMsgTxt("Convolution error: Unknown dim.");
	}
}

void convolve1D5(float *outimg, double *inpimg, int dim1, int dim2, float *filter, int orientation, int padding = 0)
{
	//int filterradiusL = (dimfilter-1) / 2;
	//int filterradiusR = (dimfilter-1) - filterradiusL;
	const int filterradiusL = 2+padding;
	const int filterradiusR = 2+padding;
	
	if(orientation == FIRSTDIM) {
		for(int j = 0; j < dim2; j++) {
			for(int i = 0; i < dim1; i++) {
				if(i < filterradiusL || i > dim1-1-filterradiusR) {
					outimg[j*dim1 + i] = 0;
				} else {
					outimg[j*dim1 + i] = 
							inpimg[j*dim1 + i - 2] * filter[0]
						+ inpimg[j*dim1 + i - 1] * filter[1]
						+ inpimg[j*dim1 + i    ] * filter[2]
						+ inpimg[j*dim1 + i + 1] * filter[3]
						+ inpimg[j*dim1 + i + 2] * filter[4];
				}
			}
		}
	} else if(orientation == SECDIM) {
		for(int j = 0; j < dim2; j++) {
			for(int i = 0; i < dim1; i++) {
				if(j < filterradiusL || j > dim2-1-filterradiusR) {
					outimg[j*dim1 + i] = 0;
				} else {
					outimg[j*dim1 + i] =
							inpimg[i + dim1*(j - 2)] * filter[0]
						+ inpimg[i + dim1*(j - 1)] * filter[1]
						+ inpimg[i + dim1*(j    )] * filter[2]
						+ inpimg[i + dim1*(j + 1)] * filter[3]
						+ inpimg[i + dim1*(j + 2)] * filter[4];
				}
			}
		}
	} else {
		mexErrMsgTxt("Convolution error: Unknown dim.");
	}
}

void normalize2Dvecfield(float *f1, float *f2, float *norm, int dim1, int dim2)
{
	for(int j = 0; j < dim2; j++) {
		for(int i = 0; i < dim1; i++) {
			int p = j*dim1+i;
			float mag = sqrt(f1[p]*f1[p] + f2[p]*f2[p]);
			norm[p] = mag;
			f1[p] = f1[p] / (mag+eps);
			f2[p] = f2[p] / (mag+eps);
		}
	}
}

void convolve2D5(float *outimg, float *buffimg, float *inpimg, int dim1, int dim2, float *filter1, float *filter2, int padding = 0)
{
	convolve1D5(buffimg, inpimg, dim1, dim2, filter1, FIRSTDIM, padding);
	convolve1D5(outimg, buffimg, dim1, dim2, filter2, SECDIM, padding);
}

void convolve2D5(float *outimg, float *buffimg, double *inpimg, int dim1, int dim2, float *filter1, float *filter2, int padding = 0)
{
	convolve1D5(buffimg, inpimg, dim1, dim2, filter1, FIRSTDIM, padding);
	convolve1D5(outimg, buffimg, dim1, dim2, filter2, SECDIM, padding);
}

// main function:
// takes a double color image and a bin size 
// returns HOG features
mxArray *process(const mxArray *mximage, const mxArray *mxsbin, mxArray **out2, mxArray **out3, mxArray **out4, mxArray **out5) {
	double *im = (double *)mxGetPr(mximage);
	const int *dims = mxGetDimensions(mximage);
	if (mxGetNumberOfDimensions(mximage) != 3 ||
			dims[2] != 3 ||
			mxGetClassID(mximage) != mxDOUBLE_CLASS)
		mexErrMsgTxt("Invalid input (must be double rgb image <X,Y,3>)");

	int sbin = (int)mxGetScalar(mxsbin);

	// memory for caching orientation histograms & their norms
	int blocks[2];
	blocks[0] = (int)round((double)dims[0]/(double)sbin);
	blocks[1] = (int)round((double)dims[1]/(double)sbin);
	float *hist = (float *)mxCalloc(blocks[0]*blocks[1]*18, sizeof(float));
	float *norm = (float *)mxCalloc(blocks[0]*blocks[1], sizeof(float));
	
	int zoff = dims[0]*dims[1];
	float *grad = createMxFloatArray3dim(dims[0], dims[1], 3, out2);
	
	float *xgradn = grad; //for convenience
	float *ygradn = grad+zoff;
	float *gradmag = grad+2*zoff;
	
	const int curvoricnt = 8; //using uu8, vv8
	
	// memory for HOG features
	int out[3];
	out[0] = max(blocks[0]-2, 0);
	out[1] = max(blocks[1]-2, 0);
	out[2] = 27+4+curvoricnt+4+4+4+1;
	
	mxArray *mxfeat;
	float *feat = createMxFloatArray3dim(out[0], out[1], out[2], &mxfeat);
	
	int visible[2];
	visible[0] = blocks[0]*sbin;
	visible[1] = blocks[1]*sbin;
	for (int x = 1; x < visible[1]-1; x++) {
		for (int y = 1; y < visible[0]-1; y++) {
			// first color channel
			double *s = im + min(x, dims[1]-2)*dims[0] + min(y, dims[0]-2);
			double dy = *(s+1) - *(s-1);
			double dx = *(s+dims[0]) - *(s-dims[0]);
			double v = dx*dx + dy*dy;

			// second color channel
			s += dims[0]*dims[1];
			double dy2 = *(s+1) - *(s-1);
			double dx2 = *(s+dims[0]) - *(s-dims[0]);
			double v2 = dx2*dx2 + dy2*dy2;

			// third color channel
			s += dims[0]*dims[1];
			double dy3 = *(s+1) - *(s-1);
			double dx3 = *(s+dims[0]) - *(s-dims[0]);
			double v3 = dx3*dx3 + dy3*dy3;

			// pick channel with strongest gradient
			if (v2 > v) {
				v = v2;
				dx = dx2;
				dy = dy2;
			} 
			if (v3 > v) {
				v = v3;
				dx = dx3;
				dy = dy3;
			}
			
			// snap to one of 18 orientations
			double best_dot = 0;
			int best_o = 0;
			for (int o = 0; o < 9; o++) {
				double dot = uu[o]*dx + vv[o]*dy;
				if (dot > best_dot) {
					best_dot = dot;
					best_o = o;
				} else if (-dot > best_dot) {
					best_dot = -dot;
					best_o = o+9;
				}
			}
			
			// add to 4 histograms around pixel using linear interpolation
			double xp = ((double)x+0.5)/(double)sbin - 0.5;
			double yp = ((double)y+0.5)/(double)sbin - 0.5;
			int ixp = (int)floor(xp);
			int iyp = (int)floor(yp);
			double vx0 = xp-ixp;
			double vy0 = yp-iyp;
			double vx1 = 1.0-vx0;
			double vy1 = 1.0-vy0;
			v = sqrt(v);

			if (ixp >= 0 && iyp >= 0) {
				*(hist + ixp*blocks[0] + iyp + best_o*blocks[0]*blocks[1]) += 
					vx1*vy1*v;
			}

			if (ixp+1 < blocks[1] && iyp >= 0) {
				*(hist + (ixp+1)*blocks[0] + iyp + best_o*blocks[0]*blocks[1]) += 
					vx0*vy1*v;
			}

			if (ixp >= 0 && iyp+1 < blocks[0]) {
				*(hist + ixp*blocks[0] + (iyp+1) + best_o*blocks[0]*blocks[1]) += 
					vx1*vy0*v;
			}

			if (ixp+1 < blocks[1] && iyp+1 < blocks[0]) {
				*(hist + (ixp+1)*blocks[0] + (iyp+1) + best_o*blocks[0]*blocks[1]) += 
					vx0*vy0*v;
			}
		}
	}
	
	float hfilter[] = {1,4,6,4,1};
	float vfilter[] = {-1,-2,0,2,1};
	
	mxArray *buffer;
	float *buff = createMxFloatArray3dim(dims[0], dims[1], 1, &buffer);
	
	//== Soft gradient field
	convolve2D5(xgradn, buff, im, dims[0], dims[1], hfilter, vfilter); //sxgrad
	convolve2D5(ygradn, buff, im, dims[0], dims[1], vfilter, hfilter); //sygrad
	
	normalize2Dvecfield(xgradn, ygradn, gradmag, dims[0], dims[1]); // normalize it

	
	// Now xxcurv, xycurv, yxcurv, yycurv:
	mxArray *nncurvMxArr;
	float *nncurv = createMxFloatArray3dim(dims[0], dims[1], 4, &nncurvMxArr);
	float *xxcurv = nncurv;
	float *xycurv = nncurv+zoff;
	float *yxcurv = nncurv+2*zoff;
	float *yycurv = nncurv+3*zoff;
	
	//== X -gradient field
	//difference along dims[1] = horizontal
	convolve2D5(xxcurv, buff, xgradn, dims[0], dims[1], hfilter, vfilter); //xx_curv
	//difference along dims[0] = vertical
	convolve2D5(xycurv, buff, xgradn, dims[0], dims[1], vfilter, hfilter); //xy_curv
	
	//== Y -gradient field
	//difference along dims[1] = horizontal
	convolve2D5(yxcurv, buff, ygradn, dims[0], dims[1], hfilter, vfilter); //yx_curv
	//difference along dims[0] = vertical
	convolve2D5(yycurv, buff, ygradn, dims[0], dims[1], vfilter, hfilter); //yy_curv

	float *pixcurv = createMxFloatArray3dim(dims[0], dims[1], 3, out3);
	float *xpixcurv = pixcurv; //for convenience
	float *ypixcurv = pixcurv+zoff; //for convenience
	float *pixcurvmag = pixcurv+2*zoff;
	
	// Now Determine gradient change in direction of the tangent
	for (int x = 0; x < dims[1]; x++) {
		for (int y = 0; y < dims[0]; y++) {
			// tc = (-xx_curv.*y_grad_norm + xy_curv .* x_grad_norm).* -y_grad_norm
			//    + (-yx_curv.*y_grad_norm + yy_curv .* x_grad_norm).*x_grad_norm;
			//tc = xx_curv.*y_grad_norm.*y_grad_norm + x_grad_norm.*y_grad_norm.*(-xy_curv-yx_curv) + (yy_curv .* x_grad_norm.*x_grad_norm);
			int p = x*dims[0] + y;
			
			float curv = xxcurv[p] * ygradn[p]*ygradn[p]
			          + (-xycurv[p]-yxcurv[p]) * xgradn[p]*ygradn[p]
			          + yycurv[p] * xgradn[p]*xgradn[p];
			xpixcurv[p] = curv * xgradn[p];
			ypixcurv[p] = curv * ygradn[p];
			pixcurvmag[p] = curv;
		}
	}
	
	// 9 orientation bins
	
	float magbinthresh[] = {3, 8, 20, 100}; // in corresp. bin, if lower than the threshold (and higher than previous)
	int nummbins = NELEMS(magbinthresh);
	
	
	float *curvhist = createMxFloatArray3dim(blocks[0], blocks[1], curvoricnt+4, out4);
	float *curvnorm = createMxFloatArray3dim(blocks[0], blocks[1], 2, out5); //norm for oris and mags
	float *curvnormori = curvnorm;
	float *curvnormmag = curvnorm + blocks[0]*blocks[1];
	
	const int pad = 4;
	
	// Make per-pixel curvature contribute to bins
		for (int x = 1; x < visible[1]-1; x++) {
		for (int y = 1; y < visible[0]-1; y++) {
			int xlp = max(min(x, dims[1]-1-pad), pad);
			int ylp = max(min(y, dims[0]-1-pad), pad);
			int p = xlp*dims[0] + ylp;
			
			// Curvature Magnitude Bin:
			int mbin = 0;
			for(;mbin < nummbins; mbin++) {
				if(abs(pixcurvmag[p]) < magbinthresh[mbin] || mbin == nummbins-1) break;
			}
			
			float dx = xpixcurv[p];
			float dy = ypixcurv[p];
			float v = gradmag[p];
			
			
			// snap to one of 18 orientations
			double best_dot = 0;
			int best_o = 0;
			for (int o = 0; o < curvoricnt; o++) {
				double dot = uu8[o]*dx + vv8[o]*dy;
				if (dot > best_dot) {
					best_dot = dot;
					best_o = o;
				}
			}
			
			// add to 4 histograms around pixel using linear interpolation
			double xp = ((double)x+0.5)/(double)sbin - 0.5;
			double yp = ((double)y+0.5)/(double)sbin - 0.5;
			int ixp = (int)floor(xp);
			int iyp = (int)floor(yp);
			double vx0 = xp-ixp;
			double vy0 = yp-iyp;
			double vx1 = 1.0-vx0;
			double vy1 = 1.0-vy0;

			if (ixp >= 0 && iyp >= 0) {
				*(curvhist + ixp*blocks[0] + iyp + best_o*blocks[0]*blocks[1]) += vx1*vy1*v; //curv ori
				*(curvhist + ixp*blocks[0] + iyp + (curvoricnt+mbin)*blocks[0]*blocks[1]) += vx1*vy1*v; //curv mag
			}

			if (ixp+1 < blocks[1] && iyp >= 0) {
				*(curvhist + (ixp+1)*blocks[0] + iyp + best_o*blocks[0]*blocks[1]) += vx0*vy1*v; //curv ori
				*(curvhist + (ixp+1)*blocks[0] + iyp + (curvoricnt+mbin)*blocks[0]*blocks[1]) += vx0*vy1*v; //curv mag
			}

			if (ixp >= 0 && iyp+1 < blocks[0]) {
				*(curvhist + ixp*blocks[0] + (iyp+1) + best_o*blocks[0]*blocks[1]) += vx1*vy0*v; //curv ori
				*(curvhist + ixp*blocks[0] + (iyp+1) + (curvoricnt+mbin)*blocks[0]*blocks[1]) += vx1*vy0*v; //curv mag
			}

			if (ixp+1 < blocks[1] && iyp+1 < blocks[0]) {
				*(curvhist + (ixp+1)*blocks[0] + (iyp+1) + best_o*blocks[0]*blocks[1]) += vx0*vy0*v; //curv ori
				*(curvhist + (ixp+1)*blocks[0] + (iyp+1) + (curvoricnt+mbin)*blocks[0]*blocks[1]) += vx0*vy0*v; //curv mag
			}
			
		}
		}
	
	// compute energy in each curvature block by summing over orientations and summing over magnitudes
	for (int o = 0; o < curvoricnt; o++) {
		float *src = curvhist + o*blocks[0]*blocks[1];
		float *dst = curvnormori;
		float *end = curvnormori + blocks[0]*blocks[1];
		while (dst < end) {
			*(dst++) += (*src) * (*src);
			src++;
		}
	}
	for (int o = 0; o < 4; o++) {
		float *src = curvhist + (o+curvoricnt)*blocks[0]*blocks[1];
		float *dst = curvnormmag;
		float *end = curvnormmag + blocks[0]*blocks[1];
		while (dst < end) {
			*(dst++) += (*src) * (*src);
			src++;
		}
	}
	
	
	// compute energy in each block by summing over orientations
	for (int o = 0; o < 9; o++) {
		float *src1 = hist + o*blocks[0]*blocks[1];
		float *src2 = hist + (o+9)*blocks[0]*blocks[1];
		float *dst = norm;
		float *end = norm + blocks[1]*blocks[0];
		while (dst < end) {
			*(dst++) += (*src1 + *src2) * (*src1 + *src2);
			src1++;
			src2++;
		}
	}

	// compute features
	for (int x = 0; x < out[1]; x++) {
		for (int y = 0; y < out[0]; y++) {
			// ====================================== HOG
			float *dst = feat + x*out[0] + y;      
			float *src, *p, n1, n2, n3, n4;

			p = norm + (x+1)*blocks[0] + y+1;
			n1 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = norm + (x+1)*blocks[0] + y;
			n2 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = norm + x*blocks[0] + y+1;
			n3 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = norm + x*blocks[0] + y;      
			n4 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);

			float t1 = 0;
			float t2 = 0;
			float t3 = 0;
			float t4 = 0;

			// contrast-sensitive features
			src = hist + (x+1)*blocks[0] + (y+1);
			for (int o = 0; o < 18; o++) {
				float h1 = min(*src * n1, 0.2);
				float h2 = min(*src * n2, 0.2);
				float h3 = min(*src * n3, 0.2);
				float h4 = min(*src * n4, 0.2);
				*dst = 0.5 * (h1 + h2 + h3 + h4);
				t1 += h1;
				t2 += h2;
				t3 += h3;
				t4 += h4;
				dst += out[0]*out[1];
				src += blocks[0]*blocks[1];
			}

			// contrast-insensitive features
			src = hist + (x+1)*blocks[0] + (y+1);
			for (int o = 0; o < 9; o++) {
				float sum = *src + *(src + 9*blocks[0]*blocks[1]);
				float h1 = min(sum * n1, 0.2);
				float h2 = min(sum * n2, 0.2);
				float h3 = min(sum * n3, 0.2);
				float h4 = min(sum * n4, 0.2);
				*dst = 0.5 * (h1 + h2 + h3 + h4);
				dst += out[0]*out[1];
				src += blocks[0]*blocks[1];
			}

			// texture features
			*dst = 0.2357 * t1;
			dst += out[0]*out[1];
			*dst = 0.2357 * t2;
			dst += out[0]*out[1];
			*dst = 0.2357 * t3;
			dst += out[0]*out[1];
			*dst = 0.2357 * t4;
			dst += out[0]*out[1];

			// ====================================== CURVATURE ORI (curvoricnt)
			p = curvnormori + (x+1)*blocks[0] + y+1;
			n1 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormori + (x+1)*blocks[0] + y;
			n2 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormori + x*blocks[0] + y+1;
			n3 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormori + x*blocks[0] + y;      
			n4 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);

			t1 = 0;
			t2 = 0;
			t3 = 0;
			t4 = 0;

			// orientation features
			src = curvhist + (x+1)*blocks[0] + (y+1);
			for (int o = 0; o < curvoricnt; o++) {
				float h1 = min(*src * n1, 0.2);
				float h2 = min(*src * n2, 0.2);
				float h3 = min(*src * n3, 0.2);
				float h4 = min(*src * n4, 0.2);
				*dst = 0.5 * (h1 + h2 + h3 + h4);
				t1 += h1;
				t2 += h2;
				t3 += h3;
				t4 += h4;
				dst += out[0]*out[1];
				src += blocks[0]*blocks[1];
			}

			// texture features
			*dst = 0.2357 * t1;
			dst += out[0]*out[1];
			*dst = 0.2357 * t2;
			dst += out[0]*out[1];
			*dst = 0.2357 * t3;
			dst += out[0]*out[1];
			*dst = 0.2357 * t4;
			dst += out[0]*out[1];
			
			// ====================================== CURVATURE MAG (4)
			p = curvnormmag + (x+1)*blocks[0] + y+1;
			n1 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormmag + (x+1)*blocks[0] + y;
			n2 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormmag + x*blocks[0] + y+1;
			n3 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);
			p = curvnormmag + x*blocks[0] + y;      
			n4 = 1.0 / sqrt(*p + *(p+1) + *(p+blocks[0]) + *(p+blocks[0]+1) + eps);

			t1 = 0;
			t2 = 0;
			t3 = 0;
			t4 = 0;

			// magnitude features
			src = curvhist + (x+1)*blocks[0] + (y+1) + (blocks[0]*blocks[1]*curvoricnt);
			for (int o = 0; o < 4; o++) {
				float h1 = min(*src * n1, 0.2);
				float h2 = min(*src * n2, 0.2);
				float h3 = min(*src * n3, 0.2);
				float h4 = min(*src * n4, 0.2);
				*dst = 0.5 * (h1 + h2 + h3 + h4);
				t1 += h1;
				t2 += h2;
				t3 += h3;
				t4 += h4;
				dst += out[0]*out[1];
				src += blocks[0]*blocks[1];
			}
			
			// texture features
			*dst = 0.2357 * t1;
			dst += out[0]*out[1];
			*dst = 0.2357 * t2;
			dst += out[0]*out[1];
			*dst = 0.2357 * t3;
			dst += out[0]*out[1];
			*dst = 0.2357 * t4;
			dst += out[0]*out[1];
			
			// truncation feature
			*dst = 0;
			
			
		}
	}

	mxFree(hist);	
	mxFree(norm);
	return mxfeat;
}

// matlab entry point
// F = features(image, bin)
// image should be color with double values
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) { 
	if (nrhs != 2)
		mexErrMsgTxt("Wrong number of inputs"); 
	if (nlhs > 5)
		mexErrMsgTxt("Wrong number of outputs");
	
	mxArray *out2, *out3, *out4, *out5;
	plhs[0] = process(prhs[0], prhs[1], &out2, &out3, &out4, &out5);
	if(nlhs > 1) plhs[1] = out2;
	if(nlhs > 2) plhs[2] = out3;
	if(nlhs > 3) plhs[3] = out4;
	if(nlhs > 4) plhs[4] = out5;
}



